@ ARM CPU Modes
.equ Usr32md, 0x10
.equ FIQ32md, 0x11
.equ IRQ32md, 0x12
.equ SVC32md, 0x13
.equ Abt32md, 0x17
.equ Und32md, 0x1b
.equ Sys32md, 0x1f

.globl contextSwitch
.global saveContext
.global loadContext

.section .text

/* void contextSwitch(ta, tb) */
contextSwitch:
	@ Almacenar dirección de tb
	stmfd sp!, {r1}

	@ R0 - ta

	@ Guardar R0-R14
	cps #Sys32md
	stmea r0, {r0-r14}
	cps #SVC32md

	@ Guardar sp_svc corregido, +4 por haber guardado tb
	mov r1, sp
	add r1, #4
	str r1, [r0, #60]

	@ Guardar lr_svc (retorno a rutina -> hdl_irq -> app)
	str lr, [r0, #64]
	
	@ Guardar spsr_svc
	mrs r1, cpsr
	str r1, [r0, #68]

	@ Guardar ttb0 (directorio de páginas)
	mrc p15, 0, r1, c2, c0, 0
	str r1, [r0, #72]
	
	// TODO falta guardar/cargar registros vfp //
	@ ===========================================

	@ Cargar pcb tarea siguiente = tb
	ldmfd sp!, {r0}

	@ Cargar ttb0
	ldr r1, [r0, #72]
	mcr p15, 0, r1, c2, c0, 0
	
	@ Flush TLB
	mov r1, #0
	mcr p15, 0, r1, c8, c7, 0

	@ Restaurar spsr
	ldr r1, [r0, #68]
	msr spsr, r1
	
	@ Restaurar lr
	ldr r1, [r0, #64]
	mov lr, r1
	
	@ Restaurar sp
	ldr r1, [r0, #60]
	mov sp, r1

	@ Restaurar registros de usuario
	cps #Sys32md
	ldmfd r0, {r0-r14}
	cps #SVC32md

	@ Volver & cargar spsr a cpsr
	movs pc, lr

@ Salva un contexto
saveContext:
	@ R0 - Task Control Block
	stmfd sp!, {r1}

	@ Guardar R0-R14
	cps #Sys32md
	stmea r0, {r0-r14}
	cps #SVC32md

	@ Guardar sp_svc corregido, +4
	mov r1, sp
	add r1, #4
	str r1, [r0, #60]

	@ Guardar lr_svc (retorno a rutina)
	str lr, [r0, #64]
	
	@ Guardar spsr_svc
	mrs r1, cpsr
	str r1, [r0, #68]

	@ Volver
	ldmfd sp!, {r1}
	mov pc, lr

@ Carga un contexto
loadContext:
	@ Restaurar spsr
	ldr r1, [r0, #68]
	msr spsr, r1
	
	@ Restaurar lr
	ldr r1, [r0, #64]
	mov lr, r1
	
	@ Restaurar sp
	ldr r1, [r0, #60]
	mov sp, r1

	@ Restaurar registros de usuario
	cps #Sys32md
	ldmfd r0, {r0-r14}
	cps #SVC32md

	@ Volver & cargar spsr a cpsr
	movs pc, lr
