#include <deltaQueue.h>

DeltaQueue* deltaQueueCreate()
{
	DeltaQueue *dQueue = new(sizeof(DeltaQueue));
	dQueue->size = 0;
	dQueue->first = 0;
	dQueue->last = 0;
	return dQueue;
}

int deltaQueueSize(DeltaQueue *dQueue)
{
	return dQueue->size;
}

void deltaQueueEnqueue(DeltaQueue *dQueue, void *data, int delta)
{
	DeltaQueueItem *item = new(sizeof(DeltaQueueItem));
	item->data = data;
	item->delta = delta;
	deltaQueuePush(dQueue, item);
}

void* deltaQueueDequeue(DeltaQueue *dQueue)
{
	DeltaQueueItem *item = deltaQueuePopFront(dQueue);
	void *data = item->data;
	delete(item);
	return data;
}

DeltaQueueItem* deltaQueueItemCreate(void *data, int delta)
{
	DeltaQueueItem *dQueueItem = new(sizeof(DeltaQueueItem));
	dQueueItem->data = data;
	dQueueItem->delta = delta;
	dQueueItem->prev = 0;
	dQueueItem->next = 0;
	return dQueueItem;
}

void deltaQueuePush(DeltaQueue *dQueue, DeltaQueueItem *item)
{
	int delta = 0;
	DeltaQueueItem *cursor = dQueue->first;

	while(cursor != 0)
	{
		delta = item->delta - cursor->delta;

		if(delta < 0)
		{
			// Me pase, poner adelante, actualizar delta del siguiente
			deltaQueueInsertBefore(dQueue, item, cursor);
			cursor->delta -= item->delta;
			return;
		}
		else
		{
			// delta positivo, actualizar delta item, comparar con siguiente
			item->delta = delta;
			cursor = cursor->next;
		}
	}

	// ubicar detras del último
	if(cursor == 0)
		deltaQueuePushBack(dQueue, item);
}

void deltaQueueInsertBefore(DeltaQueue *dQueue, DeltaQueueItem *item, DeltaQueueItem *cursor)
{
	if(cursor->prev != 0)
		((DeltaQueueItem *) cursor->prev)->next = item;
	else
		dQueue->first = item;

	item->prev = cursor->prev;
	item->next = cursor;
	cursor->prev = item;

	dQueue->size ++;
}

void deltaQueuePushBack(DeltaQueue *dQueue, DeltaQueueItem *item)
{
	if(dQueue->last != 0)
	{
		dQueue->last->next = item;
		item->prev = dQueue->last;
		dQueue->last = item;
		item->next = 0;
	}
	else
	{
		dQueue->first = item;
		dQueue->last = item;
		item->prev = 0;
		item->next = 0;
	}

	dQueue->size ++;
}

DeltaQueueItem* deltaQueueFront(DeltaQueue *dQueue)
{
	return dQueue->first;
}

DeltaQueueItem* deltaQueuePopFront(DeltaQueue *dQueue)
{
	DeltaQueueItem *item = dQueue->first;

	if(dQueue->last == dQueue->first)
		dQueue->last = 0;
	else
		((DeltaQueueItem *) item->next)->prev = 0;

	dQueue->first = dQueue->first->next;

	item->next = 0;
	dQueue->size --;
	return item;
}
