#ifndef __ELF_H__
#define __ELF_H__

#include <stdint.h>

/*
e_machine EM_ARM (40)
e_ident[EI_CLASS] ELFCLASS32
e_ident[EI_DATA] ELFDATA2LSB for little endian
                 ELFDATA2MSB for big endian
*/

#define EI_MAG0 	0 		// 0x7f
#define EI_MAG1 	1 		// 'E'
#define EI_MAG2 	2 		// 'L'
#define EI_MAG3 	3 		// 'F'
#define EI_CLASS 	4
#define EI_DATA 	5
#define EI_VERSION 	6
#define EI_PAD 		7
#define EI_NIDENT 	16 		// e_ident size

#define ELFCLASSNONE 0
#define ELFCLASS32 	1
#define ELFCLASS64 	2

#define ELFDATANONE 0
#define ELFDATA2LSB 1
#define ELFDATA2MSB 2

#define ET_NONE 	0 		// No file type
#define ET_REL 		1 		// Relocatable file
#define ET_EXEC 	2 		// Executable file
#define ET_DYN 		3 		// Shared object file
#define ET_CORE 	4 		// Core file
#define ET_LOPROC 	0xff00 	// Procesor-specific
#define ET_HIPROC 	0xffff 	// Procesor-specific

#define EM_ARM 		40 		// Machine: ARM

#define EV_NONE 	0 		// Invalid version
#define EV_CURRENT 	1 		// Current version

typedef struct Elf32_Ehdr Elf32_Ehdr;
typedef struct Elf32_Shdr Elf32_Shdr;
typedef struct Elf32_Sym Elf32_Sym;

struct Elf32_Ehdr
{
	uint8_t  e_ident[EI_NIDENT];
	uint16_t e_type;
	uint16_t e_machine;
	uint32_t e_version;
	uint32_t e_entry; 		// Entry point (vAddress)
	uint32_t e_phoff; 		// Program header table's file offset (bytes)
	uint32_t e_shoff; 		// Section header table's file offset
	uint32_t e_flags; 		// Processor-specific flags
	uint16_t e_ehsize; 		// ELF header's size (bytes)
	uint16_t e_phentsize; 	// Size in bytes of one entry in the file's program header table
	uint16_t e_phnum; 		// Number entries of program header table
	uint16_t e_shentsize; 	// Section header's size (bytes)
	uint16_t e_shnum; 		// Number entries of section header table
	uint16_t e_shstrndx; 	// Section header table index of the entry associated with section name string table
};

#define SHN_UNDEF 		0x0000 // Undefined
#define SHN_LORESERVE 	0xff00 // Lower bound of the range of reserved indexes
#define SHN_LOPROC 		0xff00 // Procesor-specific
#define SHN_HIPROC 		0xff1f // Procesor-specific
#define SHN_ABS 		0xfff1 // Absolute values not affected by realocation
#define SHN_COMMON 		0xfff2 // Common symb section
#define SHN_HIRESERVE 	0xffff // Upper bound of the range of reserved indexes

#define SHT_NULL 		0
#define SHT_PROGBITS 	1 			// Defined by program
#define SHT_SYMTAB 		2 			// Symbol table
#define SHT_STRTAB 		3 			// String table
#define SHT_RELA 		4
#define SHT_HASH 		5
#define SHT_DYNAMIC 	6 			// Symbol table
#define SHT_NOTE 		7
#define SHT_NOBITS 		8
#define SHT_REL 		9
#define SHT_SHLIB 		10
#define SHT_DYNSIM 		11
#define SHT_LOPROC 		0x70000000
#define SHT_HIPROC 		0x7fffffff
#define SHT_LOUSER 		0x80000000
#define SHT_HIUSER 		0xffffffff

#define SHF_WRITE 		0x01 		// Writable data
#define SHF_ALLOC 		0x02 		// Occupies memory during execution
#define SHF_EXECINSTR 	0x04 		// Executable machine instructions
#define SHF_MASKPROC 	0xf0000000 	// Processor-specific

struct Elf32_Shdr
{
	uint32_t sh_name;
	uint32_t sh_type;
	uint32_t sh_flags;
	uint32_t sh_addr;
	uint32_t sh_offset;
	uint32_t sh_size;
	uint32_t sh_link;
	uint32_t sh_info;
	uint32_t sh_addralign;
	uint32_t sh_entsize;
};

#define ELF32_ST_BIND(i) ((i) >> 4)
#define ELF32_ST_TYPE(i) ((i) & 0xf)
#define ELF32_ST_INFO(b,t) ((((b) << 4) + ((t) & 0xf)))

#define STB_LOCAL 0
#define STB_GLOBAL 1
#define STB_WEAK 2
#define STB_LOPROC 13
#define STB_HIPROC 15

#define STT_NOTYPE 0
#define STT_OBJECT 1
#define STT_FUNC 2
#define STT_SECTION 3
#define STT_FILE 4
#define STT_LOPROC 13
#define STT_HIPROC 15

struct Elf32_Sym
{
	uint32_t st_name;
	uint32_t st_value;
	uint32_t st_size;
	uint8_t  st_info;
	uint8_t  st_other;
	uint16_t st_shndx;
};

#endif /* __ELF_H__ */