@ ARM CPU Modes

.equ Usr32md, 0x10
.equ FIQ32md, 0x11
.equ IRQ32md, 0x12
.equ SVC32md, 0x13
.equ Abt32md, 0x17
.equ Und32md, 0x1b
.equ Sys32md, 0x1f

@ Extern declarations

.extern C_undefinedInstruction
.extern C_preFetchAbort
.extern C_dataAbort
.extern C_irq
.extern C_fiq

.extern swi_table

.globl __hdl_unf
.globl __hdl_swi
.globl __hdl_pfa
.globl __hdl_dta
.globl __hdl_irq
.globl __hdl_fiq

.section .text

.align 8

__hdl_unf:
	srsfd sp!, #SVC32md 	@ Guardar spsr & lr en el stack del modo SVC
	cps #SVC32md 			@ Cambiar a modo SVC
	stmfd sp, {r0-r14} 		@ Guardar los registros r0-r14 (modo SVC)
	sub sp, #60 			@ Corregir sp (offset 15 registros)
	mov r0, sp 				@ Mostrar excepción
	bl C_undefinedInstruction

.align 8

__hdl_swi:
	stmfd sp!, {r4, r10, r12, lr}	@ Salvar registros

	//
	mrs r10, spsr
	stmfd sp!, {r10}
	//

	ldr r10, [lr, #-4] 				@ Obtener instrucción swi
	bic r10, r10, #0xff000000 		@ Obtener swiNo

	ldr r4, =swi_table
	ldr r10, [r4, r10, LSL #2] 		@ Obtener dirección swi

	blx r10 						@ Copiar cp a lr y saltar swi

	//
	ldmfd sp!, {r10}
	msr spsr, r10
	//

	ldmfd sp!, {r4, r10, r12, pc}^ 	@ Restaurar r10, pc y volver al modo usuario

.align 8

__hdl_pfa:
	sub lr, lr, #4 			@ Corregir la dirección de retorno
	srsfd sp!, #SVC32md 	@ Guardar spsr & lr en el stack del modo SVC
	cps #SVC32md 			@ Cambiar a modo SVC
	stmfd sp, {r0-r14} 		@ Guardar los registros r0-r14 (modo SVC)
	sub sp, #60 			@ Corregir sp (offset 15 registros)
	mov r0, sp 				@ Mostrar excepción
	bl C_preFetchAbort

.align 8

__hdl_dta:
	sub lr, lr, #8 			@ Corregir la dirección de retorno
	srsfd sp!, #SVC32md 	@ Guardar spsr & lr en el stack del modo SVC
	cps #SVC32md 			@ Cambiar a modo SVC
	stmfd sp, {r0-r14} 		@ Guardar los registros r0-r14 (modo SVC)
	sub sp, #60 			@ Corregir sp (offset 15 registros)
	mov r0, sp 				@ Mostrar excepción
	bl C_dataAbort

.align 8

__hdl_irq:
	sub lr, lr, #4
	srsfd sp!, #SVC32md
	cps #SVC32md
	stmfd sp!, {r0-r3, r12, lr}
	bl C_irq
	ldmfd sp!, {r0-r3, r12, lr}
	rfe sp!

.align 8

__hdl_fiq:
	sub lr, lr, #4
	srsfd sp!, #SVC32md
	cps #SVC32md
	stmfd sp!, {r0-r3, r12, lr}
	bl C_fiq
	ldmfd sp!, {r0-r3, r12, lr}
	rfe sp!
