#include <fork.h>

segment_t* section_copy(uint ttb_copy, segment_t *section)
{
	segment_t *copy = kmalloc(sizeof(segment_t));

	copy->virt_addr = section->virt_addr;
	copy->phys_addr = 0;
	copy->offset = section->offset;
	copy->attr = section->attr;

	uint ttb = getTranslationTableBase0();
	uint src = section->virt_addr;
	uint dst = AUX_FRAME;
	uint frame;
	uint frames = (section->offset >> 12) + (((section->offset & 0xfff) > 0)? 1 : 0);

	while(frames--)
	{
		frame = mmap_alloc_user();
		
		mmu_map_smallpage(ttb, frame, dst, PAGE_RW_NA | PAGE_NOT_EXEC);
		mmu_map_smallpage(ttb_copy, frame, src, section->attr);

		for(int i = 0; i < 1024; i++)
			((uint *) dst)[i] = ((uint *) src)[i];

		mmu_unmap_smallpage(ttb, dst);

		dst += SMALL_PAGE_SIZE;
		src += SMALL_PAGE_SIZE;
	}

	return copy;
}

/*
	fork()
	======

	fork se encarga de duplicar un proceso.
*/

int fork()
{
	process_t *proc = scheduler_running();
	process_t *copy = process_create();

	// El nuevo proceso es hijo del que ejecuto fork
	copy->parent = proc;
	enqueue(proc->children, copy);

	// Mapeamos el area de kernel & IO
	copy->pcb->ttb = duplicate_ttb(proc->pcb->ttb);

	// Copio físicamente las secciones del proceso para crear
	// una proceso identico. Mapeo las secciones en el mismo espacio virtual.

	copy->text = section_copy(copy->pcb->ttb, proc->text);

	if(proc->rodata != 0)
		copy->rodata = section_copy(copy->pcb->ttb, proc->rodata);

	if(proc->data != 0)
		copy->data = section_copy(copy->pcb->ttb, proc->data);

	if(proc->bss != 0)
		copy->bss = section_copy(copy->pcb->ttb, proc->bss);

	copy->svc_stack = section_copy(copy->pcb->ttb, proc->svc_stack);
	copy->usr_stack = section_copy(copy->pcb->ttb, proc->usr_stack);


	// Si tiene sección compartida se mapea en el directrio de la copia
	if(proc->shared != 0)
		mmu_map_smallpage(copy->pcb->ttb, mmu_virt2phys(proc->pcb->ttb, proc->shared->virt_addr), proc->shared->virt_addr, proc->shared->attr);

	// Inicializar heap de proceso
	heapInit(copy->pcb->ttb, PROCESS_HEAP);

	// debug("fork : from pid(%d) to pid(%d)\n\r", proc->pid, copy->pid);
	// debug("%% process(%d) %%\n", copy->pid);
	// debug_ttb(copy->pcb->ttb);

	saveContext(copy->pcb);

	if(scheduler.running->pid != copy->pid)
	{
		// Si soy quién ejecuto fork ...
	 	scheduler_admit(copy);
	 	return copy->pid;
	}

	// Si soy la copia ...
	return 0;
}
