#ifndef __IRQ_CONTROLLER_H__
#define __IRQ_CONTROLLER_H__

#include <stdint.h>

typedef union PendingBasic
{
	struct
	{
		uint32_t arm_timer:1;
		uint32_t arm_mailbox:1;
		uint32_t arm_doorbell_0:1;
		uint32_t arm_doorbell_1:1;
		uint32_t gpu_0_halted:1;
		uint32_t gpu_1_halted:1;
		uint32_t illegal_access_type_0:1;
		uint32_t illegal_access_type_1:1;
		uint32_t pending_1:1;
		uint32_t pending_2:1;
		// GPU IRQ's; //
		uint32_t padding:22;

	} irq __attribute__((__packed__));
	uint32_t value;

} PendingBasic;

typedef union Pending1
{
	struct
	{
		uint32_t timer_0:1;
		uint32_t timer_1:1;
		uint32_t timer_2:1;
		uint32_t timer_3:1;
		uint32_t codec_0:1;
		uint32_t codec_1:1;
		uint32_t codec_2:1;
		uint32_t vc_jpeg:1;
		uint32_t isp:1;
		uint32_t vc_usb:1;
		uint32_t vc_3d:1;
		uint32_t transposer:1;
		uint32_t multicore_sync_0:1;
		uint32_t multicore_sync_1:1;
		uint32_t multicore_sync_2:1;
		uint32_t multicore_sync_3:1;
		uint32_t dma_0:1;
		uint32_t dma_1:1;
		uint32_t vc_dma_2:1;
		uint32_t vc_dma_3:1;
		uint32_t dma_4:1;
		uint32_t dma_5:1;
		uint32_t dma_6:1;
		uint32_t dma_7:1;
		uint32_t dma_8:1;
		uint32_t dma_9:1;
		uint32_t dma_10:1;
		uint32_t dma_11:1;
		uint32_t dma_12:1;
		uint32_t aux:1;
		uint32_t arm:1;
		uint32_t vpudma:1;

	} irq __attribute__((__packed__));
	uint32_t value;

} Pending1;

typedef union Pending2
{
	struct
	{
		uint32_t host_port:1;
		uint32_t video_scaler:1;
		uint32_t ccp2tx:1;
		uint32_t sdc:1;
		uint32_t dsi_0:1;
		uint32_t ave:1;
		uint32_t cam_0:1;
		uint32_t cam_1:1;
		uint32_t hdmi_0:1;
		uint32_t hdmi_1:1;
		uint32_t pixel_valve_1:1;
		uint32_t i2c_spi_slave:1;
		uint32_t dsi_1:1;
		uint32_t pwa_0:1;
		uint32_t pwa_1:1;
		uint32_t cpr:1;
		uint32_t smi:1;
		uint32_t gpio_0:1;
		uint32_t gpio_1:1;
		uint32_t gpio_2:1;
		uint32_t gpio_3:1;
		uint32_t vc_i2c:1;
		uint32_t vc_spi:1;
		uint32_t vc_i2spcm:1;
		uint32_t vc_sdio:1;
		uint32_t vc_uart:1;
		uint32_t slimbus:1;
		uint32_t vec:1;
		uint32_t cpg:1;
		uint32_t rng:1;
		uint32_t vc_arasan_sd_io:1;
		uint32_t avspmon:1;

	} irq __attribute__((__packed__));
	uint32_t value;

} Pending2;

typedef PendingBasic EnableBasic;
typedef PendingBasic DisableBasic;
typedef Pending1 Enable1;
typedef Pending1 Disable1;
typedef Pending2 Enable2;
typedef Pending2 Disable2;

#define IRQ_CONTROLLER_ADDR 0x2000B200

typedef struct IrqController
{
	PendingBasic pending_basic;
	Pending1 pending_1;
	Pending2 pending_2;
	uint32_t fiq_control;
	Enable1 enable_1;
	Enable2 enable_2;
	EnableBasic enable_basic;
	Disable1 disable_1;
	Disable2 disable_2;
	DisableBasic disable_basic;

} IrqController;

#endif /* __IRQ_CONTROLLER_H__ */