#include <list.h>

/*
 * List
 */

List* listCreate()
{
	List *list = new(sizeof(List));
	list->size = 0;
	list->first = 0;
	list->last = 0;
	return list;
}

void listDestroy(List *list)
{
	ListItem *cursor = list->first;
	ListItem *aux;
	
	while(cursor != 0)
	{
		aux = cursor->next;
		delete(cursor);
		cursor = aux;
	}
	
	delete(list);
}

int listSize(List *list)
{
	return list->size;
}

bool listEmpty(List *list)
{
	if(list->size > 0)
		return true;
	return false;
}

void* listBack(List *list)
{
	return list->last->data;
}

void* listFront(List *list)
{
	return list->first->data;
}

void listPushBack(List *list, void *data)
{
	ListItem *item = listItemCreate(data);
	listItemPushBack(list, item);
}

void listPushFront(List *list, void *data)
{
	ListItem *item = listItemCreate(data);
	listItemPushFront(list, item);
}

void* listPopBack(List *list)
{
	return listItemDestroy(listItemPopBack(list));
}

void* listPopFront(List *list)
{
	return listItemDestroy(listItemPopFront(list));
}

/*
 * ListItem
 */

ListItem* listItemCreate(void *data)
{
	ListItem *listItem = new(sizeof(ListItem));
	listItem->data = data;
	listItem->prev = 0;
	listItem->next = 0;
	return listItem;
}

void* listItemDestroy(ListItem *item)
{
	void *data = item->data;
	delete(item);
	return data;
}

ListItem* listItemFront(List *list)
{
	return list->first;
}

ListItem* listItemBack(List *list)
{
	return list->last;
}

void listItemPushBack(List *list, ListItem *item)
{
	if(list->last != 0)
	{
		list->last->next = item;
		item->prev = list->last;
		list->last = item;
		item->next = 0;
	}
	else
	{
		list->first = item;
		list->last = item;
		item->prev = 0;
		item->next = 0;
	}

	list->size ++;
}

void listItemPushFront(List *list, ListItem *item)
{
	if(list->first != 0)
	{
		list->first->prev = item;
		item->next = list->first;
		list->first = item;
		item->prev = 0;
	}
	else
	{
		list->first = item;
		list->last = item;
		item->prev = 0;
		item->next = 0;
	}

	list->size ++;
}

ListItem* listItemPopBack(List *list)
{
	ListItem *item = list->last;
	if(list->last == list->first)
		list->first = 0;
	else
		((ListItem *) item->prev)->next = 0;

	list->last = list->last->prev;
	item->prev = 0;
	list->size --;
	return item;
}

ListItem* listItemPopFront(List *list)
{
	ListItem *item = list->first;
	if(list->last == list->first)
		list->last = 0;
	else
		((ListItem *) item->next)->prev = 0;

	list->first = list->first->next;
	item->next = 0;
	list->size --;
	return item;
}

//~ void listUnlinkItem(List *list, ListItem *item)
//~ {
	//~ if(item->prev != 0)
	//~ {
		//~ ((ListItem *) item->prev)->next = item->next;
//~ 
		//~ if(item->next != 0)
			//~ ((ListItem *) item->next)->prev = item->prev;
		//~ else
			//~ list->last = item->prev;
	//~ }
	//~ else
	//~ {
		//~ list->first = item->next;
//~ 
		//~ if(item->next != 0)
			//~ ((ListItem *) item->next)->prev = 0;
		//~ else
			//~ list->last = 0;
	//~ }
//~ 
	//~ item->next = 0;
	//~ item->prev = 0;
//~ }
//~ 
//~ int listRemoveItem(List *list, void *data)
//~ {
	//~ ListItem *item = list->first;
//~ 
	//~ while(item->data != data)
		//~ item = item->next;
//~ 
	//~ if(item != 0)
	//~ {
		//~ listUnlinkItem(list, item);
		//~ kfree(item);
		//~ return 0;
	//~ }
//~ 
	//~ return -1;
//~ }

/*
 * ListIterator
 */

ListIterator* listIteratorCreate(List *list)
{
	ListIterator *iter = kmalloc(sizeof(ListIterator));
	iter->list = list;
	iter->cursor = list->first;
	iter->direction = LeftToRight;
	return iter;
}

bool listIteratorHasNext(ListIterator *iter)
{
	switch(iter->direction)
	{
		case(LeftToRight):
			if((iter->cursor != 0) && (iter->cursor->next != 0)) return true;
			
		case(RightToLeft):
			if((iter->cursor != 0) && (iter->cursor->prev != 0)) return true;
	}
	
	return false;
}

bool listIteratorHasPrev(ListIterator *iter)
{
	switch(iter->direction)
	{
		case(LeftToRight):
			if((iter->cursor != 0) && (iter->cursor->prev != 0)) return true;
			
		case(RightToLeft):
			if((iter->cursor != 0) && (iter->cursor->next != 0)) return true;
	}
	
	return false;
}

ListItem* listIteratorNext(ListIterator *iter)
{
	switch(iter->direction)
	{
		case(LeftToRight):
			iter->cursor = iter->cursor->next;
			break;
		
		case(RightToLeft):
			iter->cursor = iter->cursor->prev;
			break;
	}
	
	return iter->cursor;
}

ListItem* listIteratorPrev(ListIterator *iter)
{
	switch(iter->direction)
	{
		case(LeftToRight):
			iter->cursor = iter->cursor->prev;
			break;
		
		case(RightToLeft):
			iter->cursor = iter->cursor->next;
			break;
	}
	
	return iter->cursor;
}

void listIteratorReverse(ListIterator *iter)
{
	switch(iter->direction)
	{
		case(RightToLeft):
			iter->direction = RightToLeft;
			break;
		
		case(LeftToRight):
			iter->direction = LeftToRight;
			break;
	}
}

ListIteratorDirection listIteratorGetDirection(ListIterator *iter)
{
	return iter->direction;
}

void listIteratorDestroy(ListIterator *iter)
{
	kfree(iter);
}
