#include <miniUART.h>
#include <uart.h>
#include <ivt.h>
#include <gpio.h>
#include <syscall.h>
#include <mmu.h>
#include <heap.h>
#include <process.h>
#include <scheduler.h>
#include <sleep.h>

#include <process.h>

#include <PL011.h>

void systemInit()
{
	/** Mini UART setup */
	mini_uart_setup();
	
	/** Interrupt vector table */
	setInterruptHandler(__hdl_unf, UndefinedInstruction);
	setInterruptHandler(__hdl_swi, SWI);
	setInterruptHandler(__hdl_pfa, PreFetchAbort);
	setInterruptHandler(__hdl_dta, DataAbort);
	setInterruptHandler(__hdl_irq, IRQ);
	setInterruptHandler(__hdl_fiq, FIQ);
	
	/** System timer */
	systemTimerInit();

	/** Tabla de interrupciones por software */
	swi_init();

	/** Memory Division */
	mmap_init();

	/** VC */
	//~ videoInit();

	/** Memory Management Unit */
	mmu_init();

	/** System heap */
	heapInit(getTranslationTableBase0(), KERNEL_HEAP);

	/* Driver: UART */
	uart_init();

	/** Sleep */
	sleepInit();

	/** Process */
	last_pid = 0;
	process_t *kernel = process_create(); kernel->quantum = 30;
	process_t *init = process_init();

	/** Scheduler */
	scheduler_init(kernel);
	scheduler_admit(init);

	/* Mutex */
	mutex_init();

	debug("\n");

	irqEnable();
	
	// TODO: system daemon??.

	while(1)
	{
	}
}
