#include <mmap.h>

uint mmap[MMAP_SIZE];

void mmap_init()
{
	// debug("mmap_init\n");

	for(int i = 0; i < MMAP_SIZE; i++) mmap[i] = 0;
	
	mmap_allocate(0x00000000, (uint) &__kernel_end);
	// mmap_allocate(GPU, GPU_SIZE);
}

void mmap_allocate(uint address, uint offset)
{
	// debug("mmap_allocate : %x - %x\n", address, offset);

	uint frameNo = address >> 12;
	uint word = frameNo / 32;
	uint bit = frameNo % 32;
	
	offset = offset >> 12;
	
	while(offset--)
	{
		mmap[word] |= 1 << bit++;
		
		if(bit > 31)
		{
			bit = 0;
			word += 1;
		}
	}
}

uint mmap_alloc_system()
{
	uint address = -1;
	
	for(uint word = 0; word < MMAP_USER; word++)
	{
		for(uint bit = 0; bit < 32; bit++)
		{
			if(!(mmap[word] & (1 << bit)))
			{
				mmap[word] |= 1 << bit;
				address = (word * 32 + bit) << 12;
				// debug("mmap_alloc_system : %x\n", address);
				return address;
			}
		}
	}
	
	// debug("mmap_alloc_system : error\n");
	return address;
}

uint mmap_alloc_user()
{
	uint address = -1;
	
	for(uint word = MMAP_USER; word < MMAP_SIZE; word++)
	{
		for(uint bit = 0; bit < 32; bit++)
		{
			if(!(mmap[word] & (1 << bit)))
			{
				mmap[word] |= 1 << bit;
				address = (word * 32 + bit) << 12;
				// debug("mmap_alloc_user : %x\n", address);
				return address;
			}
		}
	}
	
	// debug("mmap_alloc_user : error\n");
	return address;
}

void mmap_free(uint address)
{
	uint frameNo = address >> 12;
	uint word = frameNo / 32;
	uint bit = frameNo % 32;
	
	mmap[word] &= ~(1 << bit);

	// debug("mmap_free : %x\n", address);
}

uint mmap_alloc_ttb()
{
	uint address = -1;

	for(uint word = 0; word < MMAP_USER; word++)
	{
		for(uint bit = 0; bit < 32; bit += 4)
		{
			if(!(mmap[word] & (0xf << bit)))
			{
				mmap[word] |= 0xf << bit;
				address = (word * 32 + bit) << 12;
				// debug("mmap_alloc_ttb : %x\n", address);
				return address;
			}
		}
	}

	// debug("mmap_alloc_ttb : error\n");
	return address;
}
