#ifndef __MMU_H__
#define __MMU_H__

#include <address.h>
#include <arm.h>
#include <mmap.h>
#include <type.h>

#define KERNEL_VIRT_START 	0
#define KERNEL_VIRT_END 	KERNEL_HEAP

#define FAULT 0

#define DOM_CLIENT 	1
#define DOM_MANAGER 0

#define PAGE_NA_NA 		0x200
#define PAGE_RO_NA 		0x210
#define PAGE_RW_NA 		0x010
#define PAGE_RO_RO 		0x220
#define PAGE_RW_RO 		0x020
#define PAGE_RW_RW 		0x030
#define PAGE_NON_GLOBAL 0x800
#define PAGE_SHARED 	0x400
#define PAGE_NOT_EXEC 	0x001
#define PAGE_BUFFERED 	0x004
#define PAGE_WR_BACK 	0x008
#define PAGE_WR_THROUGH 0x00C
#define SMALL_PAGE 		0x002 // Tipo entrada coarse

#define SECTION_NA_NA 		0x08000
#define SECTION_RO_NA 		0x08400
#define SECTION_RW_NA 		0x00400
#define SECTION_RO_RO 		0x08800
#define SECTION_RW_RO 		0x00800
#define SECTION_RW_RW 		0x00C00
#define SECTION_NON_GLOBAL 	0x20000
#define SECTION_SHARED 		0x10000
#define SECTION_NOT_EXEC 	0x00010
#define SECTION_BUFFERED 	0x00004
#define SECTION_WR_BACK 	0x00008
#define SECTION_WR_THROUGH 	0x0000C
#define COARSE_TABLE 		0x00001 // Tipo entrada master
#define SECTION 			0x00002 // Tipo entrada master

#define SECTION_SIZE 0x100000
#define SMALL_PAGE_SIZE 0x1000

void mmu_init();

int mmu_map_section(uint ttb, uint phys_addr, uint virt_addr, uint attributes);
// TODO: uint mmu_unmap_section(uint ttb, uint virt_addr);

int mmu_map_smallpage(uint ttb, uint phys_addr, uint virt_addr, uint attributes);
uint mmu_unmap_smallpage(uint ttb, uint virt_addr);

uint duplicate_ttb(uint ttb);

uint mmu_virt2phys(uint ttb, uint virt_addr);

void mmu_delete_ttb(uint ttb);

/* Debug */

void debug_ttb(uint ttb);

#endif
