#include <mutex.h>

Vector *mutexes;

void mutex_init()
{
	mutexes = vectorCreate();
	
	mutex_create(); // número 0, para controlar printf
}

int mutex_create()
{
	mutex_t *mutex = kmalloc(sizeof(mutex_t));

	mutex->state = MUTEX_UNLOCKED;
	mutex->owner = scheduler_running();
	mutex->queue = queueCreate();

	//enqueue(mutex->owner->mutexes, scheduler_running());
	vectorPushBack(mutexes, mutex);

	int key = (vectorSize(mutexes)-1);

	return key;
}

void mutex_adquire(int key)
{
	mutex_t *mutex = vectorAt(mutexes, key);

	if(mutex->state)
	{
		// Si esta bloqueado, mando la tarea a dormir
		process_t *task = scheduler_running();
		enqueue(mutex->queue, task);
		scheduler_block(task);
		schedule();
	}
	

	mutex->state = MUTEX_LOCKED;
}

void mutex_release(int key)
{
	process_t *task = scheduler_running() ;
	mutex_t *mutex = vectorAt(mutexes, key);
	mutex->state = MUTEX_UNLOCKED;

	// Cuando se libera un mutex, se despierta a 1 de las tareas que se
	// encuentran esperando (a este mutex en particular).
	if(queueSize(mutex->queue))
	{
		task = dequeue(mutex->queue);
		scheduler_unblock(task);
		schedule();
	}
}

// TODO: implementar mutex cuando hay irqs anidadas.

// int mutex_adquire(mutex_t *mutex)
// {
// 	uint32_t value, result, locked;
// 	while(1)
// 	{
// 		__asm__ ("ldrex %[value], [%[mutex]]" : [value]"=r"(value) : [mutex]"r"(&(mutex->state)));
// 		if(value != MUTEX_LOCKED)
// 			break;
// 		else
// 		{
// 			process_t *task = scheduler.running;
// 			debug("[db] Task(%d) is waiting the mutex\n\r", task->pid);
// 			scheduler_block(task);
// 			enqueue(mutex->queue, task);
// 			schedule();
// 		}
// 	}

// 	locked = MUTEX_LOCKED;

// 	__asm__ ("strex %[result], %[value], [%[mutex]]" : [result]"=r"(result) : [value]"r"(locked), [mutex]"r"(&(mutex->state)));

// 	debug("[db] Locking mutex: %d\n\r", result);

// 	return result; // 0 success, 1 failure
// }

// void mutex_release(mutex_t *mutex)
// {
// 	mutex->state = MUTEX_UNLOCKED;
// 	process_t *task = dequeue(mutex->queue);
// 	scheduler_unblock(task);
// 	debug("[db] Wakeup task(%d), the mutex is free!\n\r", task->pid);
// 	schedule();
// }
